#ifndef PIPELINETEST_H
#define PIPELINETEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** Pipeline test */
    
class PipelineTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static PipelineTest rc;
	return rc._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr inapi, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(inapi);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "PipelineTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "PipelineTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "PipelineTest::startTest")

	name = "PipelineTest";
	nScans = 2;
	testPoints.push_back(0);
	testPoints.push_back(1);
	testVariable=ST_MASK;
    }
    
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	// Set configuration
	APICALL(api, modifyABCDVar(ST_MASK, 3.0*(1-index)), "PipelineTest::getNextScan")

        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr request = sl->defaultScan();

	// Create scan
        Sct_SctApi::Scan_var s;
        APIRETCALL(api, s, createScan(), "PipelineTest::getNextScan")
        request->setScanCorba(s);

        APICALL(s, setNTrigs(100), "PipelineTest::getNextScan")

	// Adjust trigger
        Sct_SctApi::Trigger_var t;
        APIRETCALL(s, t, getTrigger1(), "PipelineTest::getNextScan")

	// 129 is critical to the pulse arriving correctly
        APICALL(t, softPulseL1A(140, 129), "PipelineTest::getNextScan")

	// "Command" 1 is the delay (starting at 140)
	APICALL(t, setCommIncr(1, 1), "PipelineTest::getNextScan")

	// Configure scan
	APICALL(s, configure(ST_ELAPSED, 140, 151, 1), "PipelineTest::getNextScan")

	return request;
    }

    virtual bool canFeedbackRealImpl() {
	return false;
    }

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    PipelineTest() {}
    Sct_SctApi::SctApiIPC_var api;
};
}
}
#endif //PIPELINETEST_H
