
#include "SubscriptionManagerImpl.h"

#include "SpecialSubscribeRequest.h"
#include "Sct/MultiMessageDebugStream.h"
#include "Sct/SctNames.h"
#include "Sct/Lookup.h"
#include "is/isinfo.h"
#include "ipc/partition.h"
#include <signal.h>

// we must not be oneway as we deliberately block callers activity till we have finished our business
void Sct::SubscriptionManagerImpl::registerNewIsServer(const char* serverNamE) {
  LOCK lock(m_busy);
  const std::string serverName = serverNamE;
  Sct::MultiMessageDebugStream m;
  ( m << "SubscritionManager: Registering new is server called ["
    << serverName << "]" ).flush();
  for (SpecialSubscribeRequests::const_iterator it=m_specialSubscribeRequests.begin();
       it != m_specialSubscribeRequests.end();
       ++it) {
    if ((*it)->m_serverPattern.matchesSomethingCalled(serverName)) {
      {
        Sct::MultiMessageDebugStream m(true,false,false);
        m << "Subscription manager found that the newly created sct_is_server called [" << serverName << "] matches this stored " << **it;
        m.flush();
      };
      this->launchOrdinarySubscriptionTo(serverName, **it);
    };
  };
  
};

void Sct::SubscriptionManagerImpl::launchOrdinarySubscriptionTo(const std::string & serverName, const SpecialSubscribeRequest & ssr) {
  LOCK lock(m_busy);

  try {
  const std::string & objectNameOrRegexp = ssr.m_objectPattern.content();
  const bool isRegexp = ssr.m_objectPattern.is_regexp();

  IPCPartition p(Sct::SctNames::getPartitionName());
  //ISInfoReceiver iir(p);

  MultiMessageDebugStream m;
  m << "Looking for a server called [" << serverName << "] in preparation for subscription";
  m.flush();

  //is::repository_var repository = p.lookup<is::repository>(serverName); // LESTER This would not be necessary if the server calling in provided us with a self reference .. maybe upgrade this LESTER
  is::repository_var repository = Sct::Lookup<is::repository>(serverName,p);

#warning LESTER This is only TEMPORARY! MUST CHECK STATUS OR ENCLOSE IN TRY BLOCK! AND GENERALLY COPY THINGS IN ISInfoReceiver::subscribe(4 arguments).
  
  if (CORBA::is_nil(repository)) {
    m << "Oh dear, it seems that the repository CORBA::is_nil";
    m.flush();
  } else {
    m << "Repository seems OK so subscibing!  First prepare";
    m.flush();


    const is::callback_ptr & another = ssr.m_cb;

    //is::callback_ptr another = ssr.m_cb;
    //is::callback_ptr yetAnother = is::callback::_duplicate(another);

    m << "now do actual subscribe to [" << serverName << "].["<<objectNameOrRegexp << "] where the last part is " << (isRegexp?"":"not ") << "a regexp.";
    m.flush();

    repository -> subscribe( objectNameOrRegexp.c_str(), 
                             isRegexp, 
                             another
                             );
  };
  m << "Subscription to [" << serverName << "] is complete.";
  m.flush();

  /* LESTER unfinished ISCallbackInfo::Callback iic; // = ssr.m_cb; (m_cb has one method: inform(in string info, in is::Reason reason) )
  ISInfo::Status s = iir.subscribe(serverName.c_str(),
                                   objectNameOrRegexp.c_str(),
                                   iic);
  if (s!=ISInfo::Success) {
    MultiMessageDebugStream os(true,true,true);
    os <<"Fitter Could not subscribe to IS server ["<<serverName 
       <<"] with regexp [" << objectNameOrRegexp << "]";
    os.flush();
   } else {
    MultiMessageDebugStream os(true,false,false);
    os <<"Fitter subscribed to IS server [" << serverName
       <<"] with regexp [" << objectNameOrRegexp << "]";
    os.flush();
  }
  
  {
    Sct::MultiMessageDebugStream m;
    m << "IMPLEMENT MEEEE !!!!!!! " << __FILE__ << " " << __LINE__;
    m.flush();
  }
  */

  } catch (const ipc::CacheExpired e) {
    Sct::MultiMessageDebugStream m;
    m << "ipc::CacheExported exception caught containing CorbaException with name [" << e.origin()._name() << "] in " << __FILE__ << " " << __LINE__;
    m.flush();
  } catch (const CORBA::Exception & e) {
    Sct::MultiMessageDebugStream m;
    m << "CORBA::Exception caught with name [" << e._name() << "] in " << __FILE__ << " " << __LINE__;
    m.flush();
  } catch (std::exception & e) {
    Sct::MultiMessageDebugStream m;
    m << "std::exception caught with message [" << e.what() << "] in " << __FILE__ << " " << __LINE__;
    m.flush();
  } catch (...) {
    Sct::MultiMessageDebugStream m;
    m << "Caught unknown exception in " << __FILE__ << " " << __LINE__;
    m.flush();    
  };
};

void Sct::SubscriptionManagerImpl::subscribe_special(const char* serverPattern,
                                                     const char* objectPattern,
                                                     CORBA::Boolean server_is_regular_expression,
                                                     CORBA::Boolean object_is_regular_expression,
                                                     is::callback_ptr cb) {
  LOCK lock(m_busy);
  const NameOrRegexp srv(serverPattern, server_is_regular_expression);
  const NameOrRegexp obj(objectPattern, object_is_regular_expression);
  
  SpecialSubscribeRequest * ssr = new SpecialSubscribeRequest(srv, obj, cb);

  Sct::MultiMessageDebugStream m;
  m << "SubscriptionManager recieved request to subscribe to [" << *ssr << "] ... processing request ... ";
  m.flush();

  m_specialSubscribeRequests.push_back(ssr);

  m << "SubscriptionManager finished processing subscription to [" << *ssr << "].";
  m.flush();

};

void Sct::SubscriptionManagerImpl::debugDumpSubscriptions() {
  LOCK lock(m_busy);
  Sct::MultiMessageDebugStream m;
  m << "SubscriptionManagerImpl has " << m_specialSubscribeRequests.size() << " specialSubscribeRequests registered at the moment. They are:";
  m.flush();

  {
    unsigned int i=0;
    for (SpecialSubscribeRequests::const_iterator it = m_specialSubscribeRequests.begin();
         it != m_specialSubscribeRequests.end();
         (++it, ++i)) {
      (m << i <<"\t" << (**it)).flush();
    };
  };
};

void Sct::SubscriptionManagerImpl::startCatchingSignals() {
  LOCK lock(m_busy);
  signal(SIGINT,  Sct::SubscriptionManagerImpl::acceptSignal);
  signal(SIGTERM, Sct::SubscriptionManagerImpl::acceptSignal);
};

void Sct::SubscriptionManagerImpl::acceptSignal(int signum) {
  std::cout << "Signal " << signum << " received at " << __FILE__ << " line " << __LINE__ << std::endl;
  if (signum==SIGINT || signum==SIGTERM) {
    std::cout << "Withdrawing from service due to signal " << signum << "." << std::endl;
    instance().tidyUpAndWithdrawEtc();
  };
};

void Sct::SubscriptionManagerImpl::tidyUpAndWithdrawEtc() {
  LOCK lock(m_busy);
  this->tidyUpAndWithdrawEtcNOW();
};

void Sct::SubscriptionManagerImpl::tidyUpAndWithdrawEtcNOW() {
  // Deliberately no lock!
  this->withdraw();
};
