
#ifndef SCT_SUBSCRIPTIONMANAGER_H
#define SCT_SUBSCRIPTIONMANAGER_H

#include "SubscriptionManagerImpl.fwd"

#include <boost/thread.hpp>
#include <ipc/object.h>
#include <ipc/server.h>
#include "SubscriptionManager.hh"
#include "SpecialSubscribeRequest.h"

namespace Sct {
  
  class SubscriptionManagerImpl : public IPCNamedObject<POA_SubscriptionManagerModule::SubscriptionManager>, public IPCServer {
  private:
    typedef boost::recursive_mutex::scoped_lock LOCK;
  private:
    SubscriptionManagerImpl() : 
      IPCNamedObject<POA_SubscriptionManagerModule::SubscriptionManager>("SCT",
                                                                         SubscriptionManagerModule::SubscriptionManager::instanceName) {
    };

  public:
    static SubscriptionManagerImpl & instance() {
      static SubscriptionManagerImpl * theInstance = new SubscriptionManagerImpl();
      return *theInstance;
    };

    /////////////////////////////////////////////////////////////////////////////
    // Implementation of the methods declared in the SubscriptionManager.idl
    /////////////////////////////////////////////////////////////////////////////

    virtual CORBA::Long whatNumberIsThis(CORBA::Long number) {
      // not worth locking this method!
      return number;
    };

    virtual void registerNewIsServer(const char* serverName);

    virtual void subscribe_special(const char* serverPattern,
                                   const char* objectPattern,
                                   CORBA::Boolean server_is_regular_expression,
                                   CORBA::Boolean object_is_regular_expression,
                                   is::callback_ptr cb);
    /// For debug purposes, debugDumpSubscriptions() should print to the implementations stdout information summarising the subscriptions that it is holding.
    virtual void debugDumpSubscriptions();

    virtual void shutdownNOW() {
      tidyUpAndWithdrawEtcNOW();
      this->stop(); // an IPCServer method!     
    };

    /////////////////////////////////////////////////////////////////////////////
    // Other things
    /////////////////////////////////////////////////////////////////////////////

    /// This method is required by IPCNamedObject.  See also kill() which is more forceful
    virtual void shutdown() {
      // can't think of anything to do here, but this is a pure virtual method of IPCServant that we must implement.
      LOCK lock(m_busy);
      this->shutdownNOW();
    };
    /// We need to catch things like ctrl-c otherwise we won't be able to "withdraw" ourselves!
    void startCatchingSignals();
    ~SubscriptionManagerImpl() {
      std::cout << "Withdrawing from service due to demise of self." << std::endl;
      this->tidyUpAndWithdrawEtc();
    };

    static void acceptSignal(int signum);
 private:
    void tidyUpAndWithdrawEtc();
    void tidyUpAndWithdrawEtcNOW();
    void launchOrdinarySubscriptionTo(const std::string & serverName, const SpecialSubscribeRequest & ssr);
    typedef std::list<SpecialSubscribeRequest *> SpecialSubscribeRequests;
    SpecialSubscribeRequests m_specialSubscribeRequests;
    boost::recursive_mutex m_busy;
  };

};

#endif
