
#ifndef SCT_MULTIMESSAGEDEBUGSTREAM_H
#define SCT_MULTIMESSAGEDEBUGSTREAM_H

#include <sstream>

namespace Sct {

  class MultiMessageDebugStream {
  private:
    const bool m_out;
    const bool m_err;
    const bool m_mrs;
    std::ostringstream * m_oss;
    /// renew() called only when m_oss=0 to try to make it better
    bool renew();
    void announce(const std::string & s);
  public:
    MultiMessageDebugStream(const bool out=true,
                            const bool err=true,
                            const bool mrs=true);
    ~MultiMessageDebugStream();
    void flush();
    template <class T> MultiMessageDebugStream & operator<<(const T & t) {
      if (m_oss) {
        // use m_oss
        (*m_oss) << t;
      } else {
        // try to renew:
        if (this->renew()) {
          // renew worked
          (*m_oss) << t;
        } else {
          // renew failed -- may be nearly out of memory
          // -- can only write to stdout and err, 
          // and must flush in case we lose the output!
          if (m_out) {
            std::cout << "MultiMessageDubugStreamEmergencyChunk[" << t << "]"
                      << std::flush; 
          };
          if (m_err) {
            std::cerr << "MultiMessageDubugStreamEmergencyChunk[" << t << "]"
                      << std::flush; 
          };
        };
      };

      return *this;
    };
  };

};

#endif
