#ifndef RXTHRESHOLDBASEDONCONFIGREGISTERTESTRESULT_H
#define RXTHRESHOLDBASEDONCONFIGREGISTERTESTRESULT_H

#include "TestResult.h"
#include "Sct/RangedVector.h"

namespace SctData{
  /**
     The class represents the result of a RxThresholdBasedOnConfigRegister test.	
     @author Christopher Lester
  */
  class RxThresholdBasedOnConfigRegisterTestResult : public TestResult {
  public:
    RxThresholdBasedOnConfigRegisterTestResult() throw();
	
    /**
       Constructor.
    */
    RxThresholdBasedOnConfigRegisterTestResult(unsigned int runNumber, const string& moduleName) throw();
	
    /**
       Destructor
    */
    virtual ~RxThresholdBasedOnConfigRegisterTestResult() throw();
	
    //Override
    virtual string getClassName() const throw();

    class StreamResult {
    public:
      typedef int ThresholdType;
      StreamResult() {
        setInvalid();
      }
      void setInvalid() {
        m_minErrorFreeRxThresh = NOT_YET_SET_OR_INVALID;
        m_maxErrorFreeRxThresh = NOT_YET_SET_OR_INVALID;
        m_bestRxThresh         = NOT_YET_SET_OR_INVALID;
      };
      int minErrorFreeRxThresh() const { return m_minErrorFreeRxThresh; };
      int maxErrorFreeRxThresh() const { return m_maxErrorFreeRxThresh; };
      int bestRxThresh() const { return m_bestRxThresh; };
      void setMinErrorFreeRxThresh(int val) {
        m_minErrorFreeRxThresh = (this->isAllowableRxThreshPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
      void setMaxErrorFreeRxThresh(int val) {
        m_maxErrorFreeRxThresh = (this->isAllowableRxThreshPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
      void setBestRxThresh(int val) {
        m_bestRxThresh = (this->isAllowableRxThreshPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
    private:
      static bool isAllowableRxThreshPrivateVar(const int val) {
        return val == NOT_YET_SET_OR_INVALID || (val>=MIN_LEGITIMATE_RX_THRESH && val <= MAX_LEGITIMATE_RX_THRESH);
      };
    public:
      static bool isGoodPhysicalRxThresh(const int val) {
        return val != NOT_YET_SET_OR_INVALID && val>=MIN_LEGITIMATE_RX_THRESH && val <= MAX_LEGITIMATE_RX_THRESH;
      };
    private:
      int m_minErrorFreeRxThresh;
      int m_maxErrorFreeRxThresh;
      int m_bestRxThresh;
      static const int NOT_YET_SET_OR_INVALID=-1;
      static const int MIN_LEGITIMATE_RX_THRESH=0;
      static const int MAX_LEGITIMATE_RX_THRESH=255;
    };
	
    /**
       get StreamResult @param index stream in range 0->1 
    */
    StreamResult& getStreamResult(const unsigned index);
    /**
       const get StreamResult @index stream in range 0->1
    */
    const StreamResult& getStreamResult(const unsigned index) const;
  private:
    RangedVector<StreamResult> streamResults;

  };
}// end of namespace SctData
#endif //#ifndef TIMEWALKTESTRESULT_H
