#ifndef SCTSERVICE_ARGUMENTS_H
#define SCTSERVICE_ARGUMENTS_H
#include <iostream>
#include <string>
#include <list>

namespace SctService {

/** Command line arguments available for fitting service */
class Arguments{
 public:
  class Subscription;
  /** 
      Constructor builds from command line args. 
      Afterwards derived class constructors must call parse(); 
  */
  Arguments(int argc, char** argv);
  /**
     number of worker threads
  */
  unsigned getNWorkers() const;
  /**
     set number of workers
   */
  void setNWorkers(const unsigned n);
  /**
      IS server to write output to 
  */
  std::string getOutputISServer() const;
  /**
     Set method for output data IS server
  */
  void setOutputISServer(const std::string& name);
  /** 
      list of IS servers to read input from 
   */
    
  std::list<Subscription> getInputISServers() const;
  /**
     start in recovery mode?
   */
  bool recoveryMode() const;
  /**
     option to start in recovery mode
  */
  void setRecoveryMode(const bool);
  /**
     name of this service in IPC
   */
  void setInstanceName(const std::string& value); 
  /**
     name of instance in ipc
   */
  std::string instanceName() const;
  /**
     name of ServerStatus object in IS
   */
  std::string getISStatusName() const;
  /**
     set name of ServerStatus object in IS
   */
  void setISStatusName(const std::string&);
  /** 
      function to write out initial arguments 
  */
  virtual void print(std::ostream& os) const;
  /** 
      write out argumets as string 
  */
  virtual std::string print() const;
  /** 
      function to write help message 
  */
  virtual void printHelp(std::ostream& os) const;
  /** 
      print help and return string 
  */
  virtual std::string printHelp() const;
  /** 
      destructor 
  */
  virtual ~Arguments();
  /** 
      turn c-style argument list into stl list
  */
  static std::list<std::string> makeList(int argc, char** argv);
 protected:
  /** 
      parse arguments - must be called in derrived class constructors;
  */
  void parse();
  /** 
      Function for getting next argument
      @return next argument 
      @param iterator must be valid. It gets incrimented and may end up not valid.
      @throw InvalidArgumentError if next argument dose not exist
      
  */
  std::string next(std::list<std::string>::iterator& it) const;
  std::list<std::string> m_list; //< record of list of arguments

  class Subscription{
  public:
    Subscription(std::string server, std::string regexp);
    std::string server;
    std::string regexp;
  protected:
    Subscription();
  };
 private:
  Arguments();
  unsigned m_nworkers;
  std::string m_options;
  std::string m_output;
  std::list<Subscription> m_input;
  bool m_recovery;
  std::string m_instanceName;
  std::string m_status_name;
  const static unsigned MAX_WORKERS=10;
};

}
#endif
