#include "../src/WorkerGroup.h"
#include "MyWorkerGroup.h"

#include "ipc/core.h"
#include "ObserverInterface.hh"

using namespace SctService;
using namespace boost;
using namespace std;
using namespace Sct;

/*
   IPC test program exercises many aspects of the Worker/Observer pattern.
   uses the conrete classes defined in MyWorkerGroup.h
   @author Alan Barr @date 16 July 2003
*/

/*
  IPC object listener provides interface to outside world, and creates a MyWorkerGroup
  in its constructor
*/

class Listener : public IPCNamedObject<POA_Observer::ObserverInterface, ipc::multi_thread> {
public:
    // constructor
    Listener(IPCPartition& p, char* objid)
	: IPCNamedObject<POA_Observer::ObserverInterface, ipc::multi_thread>(p, objid){}
    
    virtual void ipcPause(CORBA::Boolean value) throw(){
	workergroup.setPaused(value);
    }
    virtual void ipcAdd(CORBA::Long data){
	shared_ptr<MyClass> ptr(new MyClass(data));
	workergroup.push(ptr);
    }
    virtual void ipcStop(){
	workergroup.stop();
	this->withdraw();
    }
    virtual CORBA::Short ipcBusy(){
	unsigned n=workergroup.busy();
	return n;
    }
    virtual CORBA::Short ipcQueueSize(){
	unsigned n=workergroup.queueSize();
	return n;
    }
    virtual CORBA::Short ipcNWorkers(){
	unsigned n=workergroup.nWorkers();
	return n;
    }

    virtual void shutdown(){
      ipcStop();
    }

    // has a public worker group
    MyWorkerGroup workergroup;
};

int main(int argc, char** argv){
    setExceptionHandlers(argv[0]);
    IPCCore::init(argc, argv);
    
    //Set up the Listener over IPC:

    auto_ptr<IPCPartition> partition ( new IPCPartition("SCT") );
    auto_ptr<IPCServer> server (new IPCServer());

    Listener* listener = new Listener(*partition, "TestListener");
    
    // start the worker threads:

    listener->workergroup.go(4);

    // give it some data:
    
    for (int i=0; i<5; ++i){
	listener->workergroup.push(shared_ptr<MyClass>(new MyClass(i)) );
    }
    
    cout << "started Listener with number of workers = " << listener->workergroup.nWorkers() 
	 << " queue size = " << listener->workergroup.queueSize()
	 << " busy = " << listener->workergroup.busy() << endl;
    
    if (!listener->publish()){
      std::cerr << "Failed to publish listener" << std::endl;
    }
    server->run();
}
